@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: iso2022.c,v 1.46 2003/06/03 03:11:03 nalin Exp $"
+#ident "$Id: iso2022.c,v 1.50 2003/09/15 18:57:33 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <errno.h>
@@ -365,6 +365,7 @@ _vte_iso2022_ambiguous_width(struct _vte_iso2022_state *state)
 {
 	const char *wide_codelist[] = {
 		"big5",
+		"big5hkscs",
 		"euccn",
 		"eucjp",
 		"euckr",
@@ -1253,6 +1254,13 @@ process_cdata(struct _vte_iso2022_state *state, guchar *cdata, gsize length,
 	state->override = -1;
 	g_assert((current >= 0) && (current < G_N_ELEMENTS(state->g)));
 
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
+		fprintf(stderr, "Current map = %d (%c).\n",
+			current, (state->g[current] & 0xff));
+	}
+#endif
+
 	if (!state->nrc_enabled || (state->g[current] == 'B')) {
 		inbuf = cdata;
 		inbytes = length;
@@ -1262,9 +1270,9 @@ process_cdata(struct _vte_iso2022_state *state, guchar *cdata, gsize length,
 		outbuf = (gunichar*)buf;
 		outbytes = sizeof(gunichar) * length * 2;
 		do {
-			converted = _vte_conv(state->conv,
-					      &inbuf, &inbytes,
-					      (gchar**) &outbuf, &outbytes);
+			converted = _vte_conv_cu(state->conv,
+					         &inbuf, &inbytes,
+					         &outbuf, &outbytes);
 			stop = FALSE;
 			switch (converted) {
 			case ((size_t)-1):
@@ -1423,8 +1431,6 @@ process_control(struct _vte_iso2022_state *state, guchar *ctl, gsize length,
 	if (length >= 1) {
 		switch (ctl[0]) {
 		case '\r':  /* CR */
-			state->current = 0;
-			state->override = -1;
 			c = '\r';
 			g_array_append_val(gunichars, c);
 #ifdef VTE_DEBUG
@@ -1434,8 +1440,6 @@ process_control(struct _vte_iso2022_state *state, guchar *ctl, gsize length,
 #endif
 			break;
 		case '\n':  /* LF */
-			state->current = 0;
-			state->override = -1;
 			c = '\n';
 			g_array_append_val(gunichars, c);
 #ifdef VTE_DEBUG
@@ -1449,7 +1453,7 @@ process_control(struct _vte_iso2022_state *state, guchar *ctl, gsize length,
 			state->override = -1;
 #ifdef VTE_DEBUG
 			if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
-				fprintf(stderr, "\tSO\n");
+				fprintf(stderr, "\tSO (^N)\n");
 			}
 #endif
 			break;
@@ -1458,7 +1462,7 @@ process_control(struct _vte_iso2022_state *state, guchar *ctl, gsize length,
 			state->override = -1;
 #ifdef VTE_DEBUG
 			if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
-				fprintf(stderr, "\tSI\n");
+				fprintf(stderr, "\tSI (^O)\n");
 			}
 #endif
 			break;
@@ -1493,6 +1497,21 @@ process_control(struct _vte_iso2022_state *state, guchar *ctl, gsize length,
 						g_array_append_val(gunichars,
 								   c);
 					}
+#ifdef VTE_DEBUG
+					if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
+						fprintf(stderr, "\t");
+						for (i = 0; i < length; i++) {
+							c = (guchar) ctl[i];
+							fprintf(stderr,
+								"(%s%c)",
+								c < 0x20 ?
+								"^" : "",
+								c < 0x20 ?
+								c : c + 64);
+						}
+						fprintf(stderr, "\n");
+					}
+#endif
 					break;
 				case 'N': /* SS2 */
 					state->override = 2;
