@@ -21,7 +21,7 @@
 #ifndef vte_matcher_h_included
 #define vte_matcher_h_included
 
-#ident "$Id: matcher.h,v 1.3 2003/05/05 23:23:46 nalin Exp $"
+#ident "$Id: matcher.h,v 1.5 2003/05/21 21:42:16 nalin Exp $"
 
 #include <glib-object.h>
 
@@ -49,8 +49,8 @@ const char *_vte_matcher_match(struct _vte_matcher *matcher,
 /* Dump out the contents of a matcher, mainly for debugging. */
 void _vte_matcher_print(struct _vte_matcher *matcher);
 
-/* A gunichar-compatible giconv target, if one can be found. */
-const char *_vte_matcher_wide_encoding(void);
+/* Free a parameter array. */
+void _vte_matcher_free_params_array(GValueArray *params);
 
 G_END_DECLS
 
