@@ -16,8 +16,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtebg.c,v 1.6 2003/06/04 18:47:34 nalin Exp $"
+#ident "$Id: vtebg.c,v 1.9 2004/04/20 06:35:43 nalin Exp $"
 #include "../config.h"
+#include <stdio.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include "debug.h"
@@ -39,7 +40,10 @@ struct VteBgPrivate {
 static VteBg *singleton_bg = NULL;
 static void vte_bg_set_root_pixmap(VteBg *bg, GdkPixmap *pixmap);
 static void vte_bg_init(VteBg *bg, gpointer *klass);
+static GdkPixbuf *_vte_bg_resize_pixbuf(GdkPixbuf *pixbuf,
+					gint min_width, gint min_height);
 
+#if 0
 static const char *
 vte_bg_source_name(enum VteBgSourceType type)
 {
@@ -59,6 +63,7 @@ vte_bg_source_name(enum VteBgSourceType type)
 	}
 	return "unknown";
 }
+#endif
 
 #ifndef X_DISPLAY_MISSING
 
@@ -101,6 +106,18 @@ _vte_bg_display_sync(VteBg *bg)
 #endif
 }
 
+static gboolean
+_vte_property_get_pixmaps(GdkWindow *window, GdkAtom atom,
+			  GdkAtom *type, int *size,
+			  XID **pixmaps)
+{
+	return gdk_property_get(window, atom, GDK_TARGET_PIXMAP,
+				0, INT_MAX,
+				FALSE,
+				type, NULL, size,
+				(guchar**) pixmaps);
+}
+
 static GdkPixmap *
 vte_bg_root_pixmap(VteBg *bg)
 {
@@ -112,16 +129,9 @@ vte_bg_root_pixmap(VteBg *bg)
 	pixmap = NULL;
 	pixmaps = NULL;
 	gdk_error_trap_push();
-	if (gdk_property_get(bg->native->window,
-			     bg->native->atom,
-			     GDK_TARGET_PIXMAP,
-			     0,
-			     INT_MAX,
-			     FALSE,
-			     &prop_type,
-			     NULL,
-			     &prop_size,
-			     (guchar**) &pixmaps)) {
+	if (_vte_property_get_pixmaps(bg->native->window, bg->native->atom,
+				      &prop_type, &prop_size,
+				      &pixmaps)) {
 		if ((prop_type == GDK_TARGET_PIXMAP) &&
 		    (prop_size >= sizeof(XID) &&
 		    (pixmaps != NULL))) {
@@ -129,6 +139,16 @@ vte_bg_root_pixmap(VteBg *bg)
 			pixmap = gdk_pixmap_foreign_new_for_display(gdk_drawable_get_display(bg->native->window), pixmaps[0]);
 #else
 			pixmap = gdk_pixmap_foreign_new(pixmaps[0]);
+#endif
+#ifdef VTE_DEBUG
+			if (_vte_debug_on(VTE_DEBUG_MISC) ||
+			    _vte_debug_on(VTE_DEBUG_EVENTS)) {
+				gint pwidth, pheight;
+				gdk_drawable_get_size(pixmap,
+						      &pwidth, &pheight);
+				fprintf(stderr, "New background image %dx%d\n",
+					pwidth, pheight);
+			}
 #endif
 		}
 		if (pixmaps != NULL) {
@@ -270,10 +290,10 @@ vte_bg_init(VteBg *bg, gpointer *klass)
 /* Generate lookup tables for desaturating an image toward a given color.  The
  * saturation value is a floating point number between 0 and 1. */
 static void
-vte_bg_generate_desat_tables(const GdkColor *color, double saturation,
-			     guchar red[256],
-			     guchar green[256],
-			     guchar blue[256])
+_vte_bg_generate_desat_tables(const GdkColor *color, double saturation,
+			      guchar red[256],
+			      guchar green[256],
+			      guchar blue[256])
 {
 	int i;
 	/* Zero saturation -> exactly match the tinting color. */
@@ -366,6 +386,62 @@ vte_bg_cache_prune(VteBg *bg)
 	vte_bg_cache_prune_int(bg, FALSE);
 }
 
+/**
+ * _vte_bg_resize_pixbuf:
+ * @pixmap: a #GdkPixbuf, or NULL
+ * @min_width: the requested minimum_width
+ * @min_height: the requested minimum_height
+ *
+ * The background pixbuf may be tiled, and if it is tiled, it may be very, very
+ * small.  This function creates a pixbuf consisting of the passed-in pixbuf
+ * tiled to a usable size.
+ *
+ * Returns: a new #GdkPixbuf, unrefs @pixbuf.
+ */
+static GdkPixbuf *
+_vte_bg_resize_pixbuf(GdkPixbuf *pixbuf, gint min_width, gint min_height)
+{
+	GdkPixbuf *tmp;
+	gint src_width, src_height;
+	gint dst_width, dst_height;
+	gint x, y;
+
+	if (!GDK_IS_PIXBUF(pixbuf)) {
+		return pixbuf;
+	}
+
+	src_width = gdk_pixbuf_get_width(pixbuf);
+	src_height = gdk_pixbuf_get_height(pixbuf);
+	dst_width = (((min_width - 1) / src_width) + 1) * src_width;
+	dst_height = (((min_height - 1) / src_height) + 1) * src_height;
+	if ((dst_width == src_width) && (dst_height == src_height)) {
+		return pixbuf;
+	}
+
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_MISC) || _vte_debug_on(VTE_DEBUG_EVENTS)) {
+		fprintf(stderr, "Resizing (root?) pixbuf from %dx%d to %dx%d\n",
+			src_width, src_height, dst_width, dst_height);
+	}
+#endif
+
+	tmp = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(pixbuf),
+			     gdk_pixbuf_get_has_alpha(pixbuf),
+			     gdk_pixbuf_get_bits_per_sample(pixbuf),
+			     dst_width, dst_height);
+	for (y = 0; y < dst_height; y += src_height) {
+		for (x = 0; x < dst_width; x += src_width) {
+			gdk_pixbuf_copy_area(pixbuf,
+					     0, 0, src_width, src_height,
+					     tmp,
+					     x, y);
+		}
+	}
+
+	g_object_unref(G_OBJECT(pixbuf));
+	return tmp;
+}
+
 static void
 vte_bg_set_root_pixmap(VteBg *bg, GdkPixmap *pixmap)
 {
@@ -407,7 +483,7 @@ vte_bg_desaturate_pixbuf(GdkPixbuf *pixbuf,
 	long stride, width, height, channels, x, y;
 	guchar *pixels;
 
-	vte_bg_generate_desat_tables(tint, saturation, red, green, blue);
+	_vte_bg_generate_desat_tables(tint, saturation, red, green, blue);
 
 	stride = gdk_pixbuf_get_rowstride(pixbuf);
 	width = gdk_pixbuf_get_width(pixbuf);
@@ -598,6 +674,11 @@ vte_bg_get_pixmap(VteBg *bg,
 	pixmap = NULL;
 	mask = NULL;
 	if (GDK_IS_PIXBUF(pixbuf)) {
+		/* If the image is smaller than 256x256 then tile it into a
+		 * pixbuf that is at least this large.  This is done because
+		 * tiling a 1x1 pixmap onto the screen using thousands of calls
+		 * to XCopyArea is very slow. */
+		pixbuf = _vte_bg_resize_pixbuf(pixbuf, 256, 256);
 		gdk_pixbuf_render_pixmap_and_mask_for_colormap(pixbuf,
 							       colormap,
 							       &pixmap, &mask,
