@@ -18,7 +18,7 @@
 
 /* The interfaces in this file are subject to change at any time. */
 
-#ident "$Id: vtedraw.c,v 1.14 2003/05/28 21:53:59 nalin Exp $"
+#ident "$Id: vtedraw.c,v 1.16 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -215,11 +215,12 @@ _vte_draw_clear(struct _vte_draw *draw, gint x, gint y, gint width, gint height)
 
 void
 _vte_draw_set_text_font(struct _vte_draw *draw,
-			const PangoFontDescription *fontdesc)
+			const PangoFontDescription *fontdesc,
+			VteTerminalAntiAlias anti_alias)
 {
 	g_return_if_fail(draw->impl != NULL);
 	g_return_if_fail(draw->impl->set_text_font != NULL);
-	draw->impl->set_text_font(draw, fontdesc);
+	draw->impl->set_text_font(draw, fontdesc, anti_alias);
 }
 
 int
@@ -246,6 +247,14 @@ _vte_draw_get_text_ascent(struct _vte_draw *draw)
 	return draw->impl->get_text_ascent(draw);
 }
 
+int
+_vte_draw_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	g_return_val_if_fail(draw->impl != NULL, 1);
+	g_return_val_if_fail(draw->impl->get_char_width != NULL, 1);
+	return draw->impl->get_char_width(draw, c, columns);
+}
+
 gboolean
 _vte_draw_get_using_fontconfig(struct _vte_draw *draw)
 {
