@@ -21,18 +21,21 @@
 #ifndef vte_vtefc_h_included
 #define vte_vtefc_h_included
 
-#ident "$Id: vtefc.h,v 1.2 2003/05/05 23:23:46 nalin Exp $"
+#ident "$Id: vtefc.h,v 1.4 2004/04/20 05:16:56 nalin Exp $"
 
 #include <fontconfig/fontconfig.h>
 #include <pango/pango.h>
 #include <glib.h>
+#include "vte.h"
 
 G_BEGIN_DECLS
 
 typedef void (*_vte_fc_defaults_cb)(FcPattern *pattern, gpointer data);
 
 gboolean
-_vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
+_vte_fc_patterns_from_pango_font_desc(GtkWidget *widget,
+				      const PangoFontDescription *font_desc,
+				      VteTerminalAntiAlias antialias,
 				      GArray *pattern_array,
 				      _vte_fc_defaults_cb defaults_cb,
 				      gpointer defaults_data);
